/*
Copyright (C) 2004 Jens Luetkens <j.luetkens@limitland.de>

based on the KDE style "Plastik":

Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

based on the Keramik configuration dialog:
Copyright (c) 2003 Maksim Orlovich <maksim.orlovich@kdemail.net>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/

#include <qsettings.h>
#include <kglobal.h>
#include <klocale.h>

#include <qlabel.h>

#include "dotcurveconf.h"

extern "C"
{
	QWidget* allocate_kstyle_config(QWidget* parent)
	{
		KGlobal::locale()->insertCatalogue("kstyle_dotcurve_config");
		return new dotCURVEStyleConfig(parent);
	}
}

dotCURVEStyleConfig::dotCURVEStyleConfig(QWidget* parent): QGrid(2, parent)
{
	//Should have no margins here, the dialog provides them
	KGlobal::locale()->insertCatalogue("kstyle_dotcurve_config");
	setSpacing(5);
	QLabel*	checkColorLabel;
	QLabel*	radioColorLabel;
	QLabel* scrollBarStyleLabel;
	QLabel* tabOverlapLabel;
	
	// Reading config --------------------------------------------------------
	QSettings s;
	s.beginGroup("/dotcurvestyle/Settings");
	origDrawToolBarSeparator = s.readBoolEntry("drawToolBarSeparator", true);
	origCenterTabs = s.readBoolEntry("centerTabs", false);
	origHighlightMenu = s.readBoolEntry("highlightMenu", false);
	origStatusbarSeparator = s.readBoolEntry("statusbarSeparator", true);
	origUseCustomScrollbarColor = s.readBoolEntry("useCustomScrollbarColor", false);
	origCustomScrollbarColor = s.readNumEntry("customScrollbarColor", (int)white.rgb());
	origUseCustomBrushColor = s.readBoolEntry("useCustomBrushColor", false);
	origCustomBrushColor = s.readNumEntry("customBrushColor", (int)black.rgb());
	// -32768 is orange
	origCheckColor = s.readNumEntry("checkColor", -32768);
	origRadioColor = s.readNumEntry("radioColor", -32768);
	origScrollBarStyle = s.readEntry("scrollBarStyle", "ThreeButtonScrollBar");
	origTabOverlap = s.readNumEntry("tabOverlap", 7);
	// safe check
	if (origTabOverlap < 0) origTabOverlap = 0;
	s.endGroup();

	// Creating widgets and setting values ---------------------------------------------
	drawToolBarSeparator = new QCheckBox(i18n("Draw toolbar separator && handle"), this);
	drawToolBarSeparator->setChecked(origDrawToolBarSeparator);
	
	centerTabs = new QCheckBox(i18n("Center tabs"), this);
	centerTabs->setChecked(origCenterTabs);
	
	highlightMenu = new QCheckBox(i18n("Use highlight color for menu"), this);
	highlightMenu->setChecked(origHighlightMenu);
	
	statusbarSeparator = new QCheckBox(i18n("Draw status bar separator"), this);
	statusbarSeparator->setChecked(origStatusbarSeparator);
	
	useCustomScrollbarColor = new QCheckBox(i18n("Use custom s&crollbar color"), this);
	useCustomScrollbarColor->setChecked(origUseCustomScrollbarColor);
	customScrollbarColor = new KColorButton(origCustomScrollbarColor, this);
	customScrollbarColor->setEnabled(useCustomScrollbarColor);
	
	// Adding blank widget to maintain layout
	//new QWidget(this);
	
	useCustomBrushColor = new QCheckBox(i18n("Use custom brush color"), this);
	useCustomBrushColor->setChecked(origUseCustomBrushColor);
	customBrushColor = new KColorButton(origCustomBrushColor, this);
	customBrushColor->setEnabled(useCustomBrushColor);
			
	checkColorLabel = new QLabel(tr("Checkbox color:"), this);
	checkColor = new KColorButton(origCheckColor, this);
	
	radioColorLabel = new QLabel(tr("Radio button color:"), this);
	radioColor = new KColorButton(origRadioColor, this);
	
	scrollBarStyleLabel = new QLabel(tr("Scrollbar style:"), this);
	scrollBarStyle = new QComboBox(i18n("Scrollbar style"), this);
	scrollBarStyle->setEditable(false);
    	scrollBarStyle->insertItem("Windows style");
    	scrollBarStyle->insertItem("Platinum style");
    	scrollBarStyle->insertItem("KDE style");
    	scrollBarStyle->insertItem("Next style");
	scrollBarStyle->setCurrentItem(scrollBarItem(origScrollBarStyle));

	tabOverlapLabel = new QLabel(tr("Tab Overlap:"), this);
	tabOverlap = new QSpinBox(0, 20, 1, this);
	tabOverlap->setValue(origTabOverlap);
	
	// Connecting changes
	connect(drawToolBarSeparator, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
	connect(centerTabs, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
	connect(highlightMenu, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
	connect(statusbarSeparator, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
	connect(useCustomScrollbarColor, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
	connect(customScrollbarColor, SIGNAL( changed(const QColor &) ), SLOT( updateChanged() ) );
	connect(useCustomBrushColor, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
	connect(customBrushColor, SIGNAL( changed(const QColor &) ), SLOT( updateChanged() ) );
	connect(checkColor, SIGNAL( changed(const QColor &) ), SLOT( updateChanged() ) );
	connect(radioColor, SIGNAL( changed(const QColor &) ), SLOT( updateChanged() ) );
	connect(scrollBarStyle, SIGNAL( activated(int) ), SLOT( updateChanged() ) );
	connect(tabOverlap, SIGNAL( valueChanged(int) ), SLOT( updateChanged() ) );
	
	// Provide visual feedback about the state of useCustomBrushColor and useCustomScrollbarColor
	connect(useCustomBrushColor, SIGNAL( toggled(bool) ), customBrushColor, SLOT( setEnabled(bool) ) );
	connect(useCustomScrollbarColor, SIGNAL( toggled(bool) ), customScrollbarColor, SLOT( setEnabled(bool) ) );
}

dotCURVEStyleConfig::~dotCURVEStyleConfig()
{
}


void dotCURVEStyleConfig::save()
{
	QSettings s;
	s.beginGroup("/dotcurvestyle/Settings");
	s.writeEntry("drawToolBarSeparator", drawToolBarSeparator->isChecked());
	s.writeEntry("centerTabs", centerTabs->isChecked());
	s.writeEntry("highlightMenu", highlightMenu->isChecked());
	s.writeEntry("statusbarSeparator", statusbarSeparator->isChecked());
	s.writeEntry("useCustomScrollbarColor", useCustomScrollbarColor->isChecked());
	s.writeEntry("customScrollbarColor", (int)customScrollbarColor->color().rgb());
	s.writeEntry("useCustomBrushColor", useCustomBrushColor->isChecked());
	s.writeEntry("customBrushColor", (int) customBrushColor->color().rgb());
	s.writeEntry("checkColor", (int) checkColor->color().rgb());
	s.writeEntry("radioColor", (int) radioColor->color().rgb());
	s.writeEntry("scrollBarStyle", scrollBarType(scrollBarStyle->currentItem()));
	s.writeEntry("tabOverlap", tabOverlap->value());
	s.endGroup();
}

void dotCURVEStyleConfig::defaults()
{
	drawToolBarSeparator->setChecked(true);
	centerTabs->setChecked(false);
	highlightMenu->setChecked(false);
	statusbarSeparator->setChecked(true);
	useCustomScrollbarColor->setChecked(false);
	customScrollbarColor->setColor(white);
	useCustomBrushColor->setChecked(false);
	customBrushColor->setColor(black);
	//rgb: 255, 128, 0 is orange
	checkColor->setColor(QColor(255, 128, 0));
	radioColor->setColor(QColor(255, 128, 0));
	scrollBarStyle->setCurrentItem(2);
	tabOverlap->setValue(7);
	//updateChanged would be done by setChecked already
}

void dotCURVEStyleConfig::updateChanged()
{
	if (
		(drawToolBarSeparator->isChecked() == origDrawToolBarSeparator) &&
		(centerTabs->isChecked() == origCenterTabs) &&
		(highlightMenu->isChecked() == origHighlightMenu) &&
		(statusbarSeparator->isChecked() == origStatusbarSeparator) &&
		
		(useCustomScrollbarColor->isChecked() == origUseCustomScrollbarColor) &&
		(customScrollbarColor->color().rgb() == origCustomScrollbarColor.rgb()) &&
		
		(useCustomBrushColor->isChecked() == origUseCustomBrushColor) &&
		(customBrushColor->color().rgb() == origCustomBrushColor.rgb()) &&
		(checkColor->color().rgb() == origCheckColor.rgb()) &&
		(radioColor->color().rgb() == origRadioColor.rgb()) &&
		(scrollBarStyle->currentText() == origScrollBarStyle) &&
		(tabOverlap->value() == origTabOverlap)
	   )
		emit changed(false);
	else
		emit changed(true);
}

QString dotCURVEStyleConfig::scrollBarType( int listnr )
{
	switch ( listnr ) {
		case 0:
			return QString("WindowsStyleScrollBar");
		case 1:
			return QString("PlatinumStyleScrollBar");
		case 2:
			return QString("ThreeButtonScrollBar");
		case 3:
			return QString("NextStyleScrollBar");
		default:
			return QString("ThreeButtonScrollBar");
	}
}

int dotCURVEStyleConfig::scrollBarItem( QString kSBType )
{
	if( !strcmp(kSBType,"WindowsStyleScrollBar") )
		return 0;
	else if( !strcmp(kSBType,"PlatinumStyleScrollBar") )
		return 1;
	else if( !strcmp(kSBType,"NextStyleScrollBar") )
		return 3;
	else
		return 2;
}

#include "dotcurveconf.moc"
