/*
 * $Id: dotcurve.h,v 1.11 2002/02/28 11:49:08 mcosta Exp $
 * Based on dotnet: 
 *
 *        Copyright 2001, Chris Lee <lee@azsites.com>
 *        Originally copied from the KDE3 HighColor style, modified to fit mine.
 *
 *        This library is free software; you can redistribute it and/or
 *        modify it under the terms of the GNU Library General Public
 *        License version 2 as published by the Free Software Foundation.
 *
 *        This library is distributed in the hope that it will be useful,
 *        but WITHOUT ANY WARRANTY; without even the implied warranty of
 *        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *        Library General Public License for more details.
 *
 *        You should have received a copy of the GNU Library General Public License
 *        along with this library; see the file COPYING.LIB.  If not, write to
 *        the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *        Boston, MA 02111-1307, USA.
 */

#ifndef __DOTCURVE_H
#define __DOTCURVE_H

#include <kstyle.h>
#include <qbitmap.h>


//const QPointArray
#define ROUNDCORNERD 16
#define MARGIN 3
#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)


class dotCURVEstyle : public KStyle
{
        Q_OBJECT

public:
        dotCURVEstyle();
        virtual ~dotCURVEstyle();

	void renderMenuBlendPixmap( KPixmap &pix, const QColorGroup &cg, const QPopupMenu *popup ) const;
	
	void drawKStylePrimitive( KStylePrimitive kpe,
                QPainter* p,
                const QWidget* widget,
                const QRect &r,
                const QColorGroup &cg,
                SFlags flags = Style_Default,
                const QStyleOption& = QStyleOption::Default ) const;

        void drawPrimitive( PrimitiveElement pe,
                QPainter *p,
                const QRect &r,
                const QColorGroup &cg,
                SFlags flags = Style_Default,
                const QStyleOption &opt = QStyleOption::Default ) const;

        void drawControl( ControlElement element,
                QPainter *p,
                const QWidget *widget,
                const QRect &r,
                const QColorGroup &cg,
                SFlags flags = Style_Default,
                const QStyleOption& = QStyleOption::Default ) const;

        void drawComplexControl( ComplexControl control,
                QPainter *p,
                const QWidget *widget,
                const QRect &r,
                const QColorGroup &cg,
                SFlags flags = Style_Default,
                SCFlags controls = SC_All,
                SCFlags active = SC_None,
                const QStyleOption& = QStyleOption::Default ) const;

        int pixelMetric( PixelMetric m,
                const QWidget *widget = 0 ) const;

        QRect subRect( SubRect r,
                const QWidget *widget ) const;

        QSize sizeFromContents(QStyle::ContentsType t,
                               const QWidget *w,
                               const QSize &s,
                               const QStyleOption &o) const;

        void polish(QWidget* widget);
        void polish(QApplication* app);
        int styleHint ( StyleHint stylehint,
                        const QWidget * widget = 0,
                        const QStyleOption & opt = QStyleOption::Default,
                        QStyleHintReturn * returnData = 0 ) const;
        void renderGripper(QPainter *p,
                           int x, int y, int w, int h,
                           bool horizontal = true) const;


protected:
        void renderButton(QPainter *p,
                          const QRect &r,
                          const QColorGroup &g,
                          bool sunken = false) const;

private:
// Disable copy constructor and = operator
        dotCURVEstyle( const dotCURVEstyle & );
        dotCURVEstyle& operator=( const dotCURVEstyle & );

        void cleanSides (QPainter* p,
                         const int x, const int y, const int w, const int h,
                         const QColor c,
                         const bool horiz = true) const 
        {
                if ( horiz ) {
                        p->fillRect(x, y, ROUNDCORNERD/2, h, c);
                        p->fillRect(x + w - ROUNDCORNERD/2, y, ROUNDCORNERD/2, h, c);
                } else {
                        p->fillRect(x, y, w, ROUNDCORNERD/2, c);
                        p->fillRect(x, y + h - ROUNDCORNERD/2, w, ROUNDCORNERD/2, c);
                }
        }
        void renderRoundRect(QPainter* p,
                             const int x, const int y, const int w, const int h,
                             bool sharpCorner = false) const;

	bool 		kickerMode, khtml;	
};

#endif

// vim: set noet ts=4 sw=4:
